package Requester::fileaccess;

use strict;
use FileHandle;
use Requester::Requester;

our(@ISA);
@ISA = qw(Requester::Requester);

use MicroTime;

my $fileaccess = new Requester::fileaccess();

# constants
my $status_ok = 3;
my $default_filesize = 64 * 1024;
my $timing_threshold = 0;

# instance variables
my $status = undef;
my $statusmsg = undef;

sub new
{
   my( $this, $args ) = @_;
   my $class = ref($this) || $this;
   my $self = {};
   bless( $self, $class );
   $self->register("fileaccess");
   return( $self );
}


sub getinfo {
    my( $self, $vars, $indexes ) = @_;

    # we cannot walk the fileaccess domain
    return undef unless( $indexes );
		
		my $id = $self->{"env"}->expand_value( "id" );
		print "instance id: $id\n" if ($BigSister::common::dl > 6);
		
		my $file = $self->{"env"}->expand_value( "path" );
		print "target file: $file\n" if ($BigSister::common::dl > 4);
		
		# everything is ok until now ..
		$status = $status_ok;
		$statusmsg = "Ok";
				
		return undef unless ( $vars );
		my $var = $vars->[0];
		print "requesting $var\n" if ($BigSister::common::dl > 4);

		# initialize result hash
		my $info = {};
		$info -> {$id} -> {"path"} = $file;
		
		
		if ( $var =~ /accesstime/ ) {
			my ($accesstime) = accesstime($file);
			if (defined $accesstime) {
				$info -> {$id} -> {"accesstime"} = $accesstime;
			}
			
			# cleanup
			if ( -f $file ) {
				unlink($file);
			}
		}
		
		my $length = 0;
		if ( $var =~ /writebandwidth/ ) {
			$length = $self->{"env"}->expand_value( "size" );
			my ($writebandwidth) = writefile($file, $length);
			if (defined $writebandwidth) {
				$info -> {$id} -> {"writebandwidth"} = $writebandwidth;
			}
			
			# cleanup
			if ( -f $file ) {
				unlink($file);
			}
		}
		
		if ( $var =~ /readbandwidth/ ) {
			$length = $self->{"env"}->expand_value( "size" );
			my ($readbandwidth) = readfile($file, $length);
			if (defined $readbandwidth) {
				$info -> {$id} -> {"readbandwidth"} = $readbandwidth;
			}
		}
		
		# return path, status, statusmsg
		$info -> {$id} -> {"path"} = $file;
		$info -> {$id} -> {"status"} = $status;
		$info -> {$id} -> {"statusmsg"} = $statusmsg;		
		
		return $info;
}

sub accesstime() {
	my ($file) = @_;

	# we will not overwrite an existing file
	if ( -f $file ) {
			$status = 1;
			$statusmsg = "File $file exists, entire test will be skipped";
			return;
	}
	
	my $accesstime = undef;	
	my $start = MicroTime::gettimeofday();
	my $wfh = new FileHandle();
	if ( $wfh -> open("> $file") ) {
		$accesstime = MicroTime::difftime($start, MicroTime::gettimeofday());
		if ( $accesstime > $timing_threshold ) {
			print "accesstime is: $accesstime\n" if ($BigSister::common::dl > 4);
		} else {
			$statusmsg = "Time difference too small for an accurate result... lucky you!";
			return;
		}
		$wfh -> close();
		
	} else {
		$status = 1;
		$statusmsg = "Failed to open file $file: ".$!;
		return;
	}
	
	return ($accesstime);	
}


sub writefile() {
	my ($file, $size) = @_;
	
	# we will not overwrite an existing file
	if ( -f $file ) {
			$status = 1;
			$statusmsg = "File $file exists, entire test will be skipped";
			return;
	}
	
	# and we do not accept negative or zero filesize
	if ( $size <= 0 ) {
		$status = 1;
		$statusmsg = "Filesize must be > 0";
		return;
	}
	
	my $bandwidth = undef;

	my $wfh = new FileHandle();
	if ( $wfh -> open("> $file") ) {
		# we create a buffer, buffersize is 32 kbytes
		my $buffsize = 1024*32;
		my $buff;
		for (my $index = 0; $index < $buffsize/2; $index++) {
			$buff .= pack("S", $index);
		}
			
		my $processedbytes = 0;
		my $start = MicroTime::gettimeofday();
		while ($processedbytes < $size) {
			my $written;
			if (($size - $processedbytes) > $buffsize) {
				$written = syswrite($wfh,$buff,$buffsize);
			} else {
				$written = syswrite($wfh,$buff,($size - $processedbytes));
			}
			if ( defined $written ) {
				$processedbytes += $written;
			} else {
				$status = 1;
				$statusmsg = "Error during write operation after $processedbytes bytes: ".$!;
					
				# clean up before returning ..
				$wfh -> close();
				return;
			}
		}
		eval { $wfh->sync(); };

		my $writetime = MicroTime::difftime($start, MicroTime::gettimeofday());
		print "writetime is: $writetime\n" if ($BigSister::common::dl > 6);
		$wfh -> close();
		
		if ( $writetime > $timing_threshold ) {
			$bandwidth = $size/$writetime;
			print "write bandwidth: $bandwidth (size: $size)\n" if ($BigSister::common::dl > 4);
		} else {
			$statusmsg = "Time difference too small, unable to calculate a reliable result.";
			return;
		}		
	} else {
		$status = 1;
		$statusmsg = "Failed to open file $file for writing: ".$!;
		return;
	}
	
	return ($bandwidth);
}

sub readfile() {
	my ($file, $length) = @_;
	
	# the file must exist
	unless ( -f $file ) {
			$status = 1;
			$statusmsg = "File $file does not exist, entire test will be skipped";
			return;
	}
	
	# and we do not accept negative or zero length
	if ( $length <= 0 ) {
		$status = 1;
		$statusmsg = "Read length must be > 0";
		return;
	}
	
	# ... finally, the file must be big enough
	my $filesize = (stat($file))[7];
	print "file: $file, size: $filesize\n" if ($BigSister::common::dl > 6);
	
	if ( $filesize < $length ) {
		$status = 1;
		$statusmsg = "File: $file, size is only $filesize bytes (min length expected: $length bytes)";
		return
	}
	
	my $bandwidth = undef;
	my $rfh = new FileHandle();
	if ( $rfh -> open("< $file") ) {
	
		# we create a buffer, buffersize is 32 kbytes
		my $buffsize = 1024*32;
		my $buff;
		for (my $index = 0; $index < $buffsize/2; $index++) {
			$buff .= pack("S", $index);
		}
		
		
		# read data from file
		my $processedbytes = 0;
		my $start = MicroTime::gettimeofday();
		while ($processedbytes < $length) {
			my $read;
			if (($length - $processedbytes) > $buffsize) {
				$read = sysread($rfh,$buff,$buffsize);
			} else {
				$read = sysread($rfh,$buff,($length - $processedbytes));
			}
			if ( defined $read ) {
				$processedbytes += $read;
			} else {
				$status = 1;
				$statusmsg = "Error during read operation after $processedbytes bytes: ".$!;
				return;
			}
		}
		$rfh -> close();
		
		my $readtime = MicroTime::difftime($start, MicroTime::gettimeofday());
		if ( $readtime > $timing_threshold ) {
			$bandwidth = $length/$readtime;
			print "read bandwidth: $bandwidth\n" if ($BigSister::common::dl > 4);
		} else {
			$statusmsg = "Time difference too small, unable to calculate a reliable result.";
			return;
		}
			
	} else {
		$status = 1;
		$statusmsg = "Failed to open file $file for reading: ".$!;
		return;
	}
	
	return ($bandwidth);	
}

1;
